//   |------------------------------------------------------------|
//   | Basic Seismic Utilities (BSU)                              |
//   | LANGUAGE: Scilab                                           |
//   | AUTHOR: P. MICHAELS                      DATE: 16 Apr 2002 |
//   | FUNCTION NAME: delaytmR.sci                                |
//   | DIRECTORY: BSUROOT/Scilab                                  |
//   |------------------------------------------------------------|
//   | purpose: compute solution from reciprocal shooting         |
//   |   (common receiver gathers formed from common shot gathers)|
//   |------------------------------------------------------------|
//   | Scilab is an interpreter useful for scientifc calculations.|
//   | It is similar to MATLAB and may be obtained free from      |
//   | http://www-rocq.inria.fr/scilab                            |
//   | Scilab is available for many platforms, including LINUX    |
//   |------------------------------------------------------------|
//   |  NOTICE:                                                   |
//   |  Copyright (c) 2002 Paul Michaels                          |
//   |  <pm@cgiss.boisestate.edu>                                 |
//   |  This program is free software; you can                    |
//   |  redistribute it and/or modify it under the terms          |
//   |  of the GNU General Public License as published            |
//   |  by the Free Software Foundation; either version           |
//   |  3 of the License, or (at your option) any later           |
//   |  version.  This program is distributed in the              |
//   |  hope that it will be useful, but                          |
//   |  WITHOUT ANY WARRANTY; without even the implied            |
//   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
//   |  PARTICULAR PURPOSE.  See the GNU General Public           |
//   |  License for more details.                                 |
//   |  You should have received a copy of the GNU                |
//   |  General Public License along with this program;           |
//   |  if not, write to the Free Software Foundation,            |
//   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
//   |------------------------------------------------------------|
// Delay Time Procedure  Modified for River Problem (water
//                       depth of source extra files, wds.dat)
//                       wds.dat=2col. (depth m, sp number)
//  Author: P. Michaels
clear
ingfile=xgetfile('G*',title='Choose G=system matrix file ')
indfile=xgetfile('D*',title='Choose d=data vector file ')
inefile=xgetfile('E*',title='Choose e=elevations file ')
wdsfile=xgetfile('*.dat',title='Choose Water Depth File (ie wds.dat) ')

//read water depth at source
fp1=file('open',wdsfile,'old');
WDS=read(fp1,-1,2);
wds=WDS(:,1);
iwds=WDS(:,2);
file('close',fp1);

//read data
fp1=file('open',indfile,'old');
D=read(fp1,-1,2);
d=D(:,1);
flag=D(:,2);
file('close',fp1);

//read elevations
fp1=file('open',inefile,'old');
E=read(fp1,-1,3);
elv=E(:,3);
chan=E(:,1);
recv=E(:,2);
file('close',fp1);

nvarbl=evstr(x_dialog(['# Number of shots ? '],['1']));
nshot=nvarbl(1);

//assume 48 channels
//nchanl=48;
clear nvarlb;
nvarbl=evstr(x_dialog(['# Number of Channels? '],['1']));
nchanl=nvarbl(1);
ncols=nshot+nchanl+1;
//read system matrix 
fp1=file('open',ingfile,'old');
G=read(fp1,-1,ncols);
file('close',fp1);

[N,M]=size(G);
//nchanl=input('Enter number of channels ')
//nshot=input('Enter Number of Shots = ')
imax=nshot+nchanl+1;

//relative weighting matrix
pi2=3.1415926/2.;
// wsmooth=input(' Enter Smoothness Weight 0<W<1 ')
nvarbl=evstr(x_dialog(['Enter Smoothness Weight 0<W<1 '],['.50']));
wsmooth=nvarbl(1);
wsmtxt=sprintf("%f",wsmooth);
wfit=1-wsmooth;
for j=(nshot+1):(M-1)
for k=(nshot+1):M
W(j,k)=0;
end
W(j,j+1)=1;
W(j,j)=-1;
end
W(M,M)=1;
for j=1:(M-1)
W(M,j)=0;
end

// Compute Smooth Inverse
H=inv(wsmooth*W'*W+wfit*G'*G)*wfit*G';

m=H*d;

for i=1:nshot
dltsht(i)=m(i);
end
k=1;
for i=(nshot+1):(imax-1)
dltrcv(k)=m(i);
k=k+1;
end
v2=1/(m(imax));
v2txt=sprintf("%d",ceil(v2));
disp(' Refractor Velocity');
ceil(v2)

disp(' Shot Delay Times');
dltsht

//error analysis
cdd=eye(N,N);
sigmat=.001;  // pics +/- uncertainty, 1 std dev (68% conf)
for j=1:N
cdd(j,j)=cdd(j,j)*sigmat*sigmat;
end
cm=H*cdd*H';
sigmaslow=sqrt(cm(imax,imax));
sigmav2=(1/(m(imax)-sigmaslow)-1/(m(imax)+sigmaslow) )/2;
k=0;
for j=(nshot+1):(imax-1)
k=k+1;
errdt(k)=sqrt(cm(j,j));
end

//  plot delay times
jrecv(1:length(dltrcv))=recv(1:length(dltrcv));
plot(jrecv,dltrcv);
xmax=max(jrecv); xmin=min(jrecv);
ymax=max(dltrcv); ymin=min(dltrcv);
xtitle(['Receiver Delay Times (V2=',v2txt,')']);

//label x-axis
labelx="Receiver Station Number";
xc=(xmin+xmax)/2;
yc=ymin-(ymax-ymin)/8;
xstring(xc,yc,labelx);

//label y-axis
labely="Delay Time";
yc=(ymin+ymax)/2 ;
xc=xmin-(xmax-xmin)/10;
xstring(xc,yc,labely);
disp('writing delay times to disk .  .  .');
fp2=file('open',['dtimes.dat'],'unknown');
AA=[jrecv, dltrcv];
write(fp2,AA,'(2x,f10.2,2x,f10.4)');
file('close',fp2)

// v1=input(' Enter overburden velocity = ')
nvarbl=evstr(x_dialog(['Overburden Velocity '],['500']));
v1=nvarbl(1);

//plot structural interpretation (overburden velocity constant)
xbasc();
theta=asin(v1/v2);
cs=cos(theta);
if v2>1500 then
theta1=asin(1500/v2);  //p-refraction problem, ray optics
else
theta1=0.             //water over sv conversion problem, no ray optics
end
cst1=cos(theta1);     //cosine of angle in water layer
//h=(v1/cs)*dltrcv;

//calculate thickness of v1 layer, h
for jsp=1:length(dltrcv)
if wds(jsp)>0. then
h(jsp)=(v1/cs)*(dltrcv(jsp)-(wds(jsp)/cst1)/1500.);
errh(jsp)=(v1/cs)*errdt(jsp);
else
h(jsp)=(v1/cs)*dltrcv(jsp);
errh(jsp)=(v1/cs)*errdt(jsp);
end
end

disp('writing errorbars to disk .  .  .');
fp2=file('open',['ebars.dat'],'unknown');
txt=sprintf(" Assumed V1=%d for Structural Interp. ",ceil(v1));
write(fp2,txt,'(a)');
txt=sprintf(" Pick Time Std_Dev=%f (68pct conf) ",sigmat);
write(fp2,txt,'(a)');
txt=sprintf(" V2=%d (+/-%d) 95 percent confidence",ceil(v2),ceil(1.96*sigmav2));
write(fp2,txt,'(a)');
txt=sprintf(" Error Bars 95pct conf ");
write(fp2,txt,'(a)');
txt=sprintf(" Receiver  Err_dt   Err_h) ");
write(fp2,txt,'(a)');
AA=[jrecv, errdt*1.96, errh ];
write(fp2,AA,'(1x,f5.0,2x,f10.4,2x,f10.2)');
file('close',fp2)

NN=length(h);
for j=1:NN
e(j)=elv(j);      //source elevation
if wds(j)>0. then
ewds(j)=e(j)-wds(j);  //elevation of river bottom
else
ewds(j)=e(j);
end
end
wx=ewds-h;          //elevation of refractor now determined below wds

// plot structural solution
xmax=max(jrecv); xmin=min(jrecv);
ymax=max(e); ymin=min(wx);
frect=[xmin,ymin,xmax,ymax];
rect=[0,0,1.,1.];
nax=[5,2,5,2];
xsetech(rect,frect,"nn");
plotframe(frect,nax);

plot2d(jrecv,wx,1,'000');
plot2d(jrecv,e,1,'000');
plot2d(iwds,ewds,1,'000');   //use iwds to avoid order errors
v1txt=sprintf("%d",ceil(v1));
xtitle(['Receiver Structure (V1=',v1txt,')']);

//label x-axis
labelx="Receiver Station Number";
xc=(xmin+xmax)/2;
yc=ymin-(ymax-ymin)/8;
xstring(xc,yc,labelx);

//label y-axis
labely="Elevation";
yc=(ymin+ymax)/2 ;
xc=xmin-(xmax-xmin)/10;
xstring(xc,yc,labely);

disp('writing structure to disk   .  .');
fp2=file('open',['struct.dat'],'unknown');
AA=[jrecv, wx, e, ewds];
write(fp2,AA,'(2x,f10.2,2x,f10.2,2x,f10.2,2x,f10.2)');
file('close',fp2)

//  h1=input(' Enter depth of refractor = ')
nvarbl=evstr(x_dialog(['Depth of refractor '],['10']));
h1=nvarbl(1);
xbasc();
//plot velocity interpretation (refractor depth=constant)
for jr=1:NN
vovb(jr)=v2/sqrt(1+ (dltrcv(jr)*(v2/h1)).^2);
end
// plot velocity solution
xmax=max(jrecv); xmin=min(jrecv);
ymax=max(vovb); ymin=min(vovb);
frect=[xmin,ymin,xmax,ymax];
rect=[0,0,1.,1.];
nax=[5,2,5,2];
xsetech(rect,frect,"nn")
plotframe(frect,nax);

plot2d(jrecv,vovb,1,'000');
h1txt=sprintf("%d",ceil(h1));
xtitle(['Receiver V1 Velocity Solution (H1=',h1txt,')']);

//label x-axis
labelx="Receiver Station Number";
xc=(xmin+xmax)/2;
yc=ymin-(ymax-ymin)/8;
xstring(xc,yc,labelx);

//label y-axis
labely="m/s";
yc=(ymin+ymax)/2 ;
xc=xmin-(xmax-xmin)/10;
xstring(xc,yc,labely);

disp('writing velocity structure to disk   .  .');
fp2=file('open',['velocity.dat'],'unknown');
clear AA;
AA=[jrecv, vovb];
write(fp2,AA,'(2x,f10.2,2x,f10.2,2x,f10.2)');
file('close',fp2)

//calculated times
dcal=G*m;

//least square error
e=d-dcal;
emin=sqrt(e'*e/length(e));

//plot time fit
clear nvarbl;
nvarbl=evstr(x_dialog(['# Constraint Equations (if any) '],['1']));
nconst=nvarbl(1);
N2=(N-nconst);

xdel();
disp('writing data times to disk   .  .');
fp2=file('open',['ttimes.dat'],'unknown');
write(fp2,nshot,'(2x,i5)');
write(fp2,nchanl,'(2x,i5)');
clear AA;
AA=zeros(N2,3);

//plot and write
kstart=1;
krow=0;
for k=1:nshot
//find start and end for current shot
for jj=kstart:N2
if G(jj,k)==1 then kend=jj; end;
end  //next jj

if k==1 then
  plot2d(flag(1:N2),d(1:N2),-3);
end;  //endif
  plot2d(flag(kstart:kend),dcal(kstart:kend),2,"000");

for jrow=kstart:kend
krow=krow+1;
AA(krow,1)=flag(jrow);
AA(krow,2)=d(jrow);
AA(krow,3)=dcal(jrow);
end //next jrow

nvalues=kend-kstart+1;
if nvalues<nchanl then
kstp=nchanl-nvalues;
for jj=1:kstp
krow=krow+1;
AA(krow,1)=flag(kend);
AA(krow,2)=d(kend);
AA(krow,3)=dcal(kend);
end  //next jj
end  //endif
kstart=kend+1;
end;  //next k

write(fp2,AA,'(2x,f10.2,2x,f10.4,2x,f10.4)');
file('close',fp2)
