//   |------------------------------------------------------------|
//   | $Id: delaytmplot.sci,v 1.6 2009-05-11 18:42:51 pm Exp $    |
//   | Basic Seismic Utilities (BSU)                              |
//   | LANGUAGE: Scilab                                           |
//   | AUTHOR: P. MICHAELS                                        |
//   | FUNCTION NAME: delaytmplot.sci                             |
//   | DIRECTORY: BSUROOT/Scilab                                  |
//   |  revised for scilab 4.0                                    |
//   |------------------------------------------------------------|
//   | purpose: plot solution from delaytm.sci                    |
//   |   (normal refraction shooting, common shot gathers)        |
//   |------------------------------------------------------------|
//   | Scilab is an interpreter useful for scientifc calculations.|
//   | It is similar to MATLAB and may be obtained free from      |
//   | http://www-rocq.inria.fr/scilab                            |
//   | Scilab is available for many platforms, including LINUX    |
//   |------------------------------------------------------------|
//   |  NOTICE:                                                   |
//   |  Copyright (c) 2009 Paul Michaels                          |
//   |  <pm@cgiss.boisestate.edu>                                 |
//   |  This program is free software; you can                    |
//   |  redistribute it and/or modify it under the terms          |
//   |  of the GNU General Public License as published            |
//   |  by the Free Software Foundation; either version           |
//   |  3 of the License, or (at your option) any later           |
//   |  version.  This program is distributed in the              |
//   |  hope that it will be useful, but                          |
//   |  WITHOUT ANY WARRANTY; without even the implied            |
//   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
//   |  PARTICULAR PURPOSE.  See the GNU General Public           |
//   |  License for more details.                                 |
//   |  You should have received a copy of the GNU                |
//   |  General Public License along with this program;           |
//   |  if not, write to the Free Software Foundation,            |
//   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
//   |------------------------------------------------------------|
//  Final Plot of delay time solution (strucure)

clear

//---load results from delaytm.sci run
fp1=file('open','struct.dat','old');
struc=read(fp1,-1,3);    // jstation, wx, elev
jstation=struc(:,1);
wx=struc(:,2);
elev=struc(:,3);
file('close',fp1);
ymax=max(elev);
ymin=min(wx);
xmin=min(jstation);
xmax=max(jstation);
frect=[xmin,ymin,xmax,ymax];

plot2d(jstation,elev,style=2,rect=frect);     // ground level
plot2d(jstation,wx,style=2,frameflag=0,axesflag=0);    //  wx (refractor)
//---load fit from delaytm.sci run
fp1=file('open','ttimes.dat','old');
VEL=read(fp1,1,2);
v1=VEL(1); v2=VEL(2);
nshot=read(fp1,1,1);
for k=1:nshot
kend(k)=read(fp1,1,1) ;
end
times=read(fp1,-1,3);    // flag, observed, calculated 
flag=times(:,1);
d=times(:,2);
dcal=times(:,3);
file('close',fp1);
v1txt=sprintf("%d",v1);
v2txt=sprintf("%d",v2);
xtitle("V1="+v1txt+"(m/s)  V2="+v2txt+"(m/s) ","Station","Elevation");
xgrid(3);

//-------------travel time fit plot
x_message('Continue?');

xdel();
kstart=1
xmin=min(flag);
xmax=max(flag);
ymin=min(min(d),min(dcal))*1000; // *1000 converts sec to ms
ymax=max(max(d),max(dcal))*1000;
frect=[xmin,ymin,xmax,ymax];
for k=1:nshot
if k==1 then
  plot2d(flag(kstart:kend(k)),d(kstart:kend(k))*1000,style=-3,rect=frect);
else;
  plot2d(flag(kstart:kend(k)),d(kstart:kend(k))*1000,style=-3,frameflag=0,axesflag=0);
end;  //endif 
  plot2d(flag(kstart:kend(k)),dcal(kstart:kend(k))*1000,style=2,frameflag=0,axesflag=0);
  kstart=kend(k)+1
end;  //next k
xtitle("O=Observed and ___=Solution ","Station","Arrival Time (ms)");
xgrid(3);
x_message('Continue?');
xdel();
fp2=file('open','velocity.dat','old')
clear flag
h1=read(fp2,1,1);
h1txt=sprintf("%d",ceil(h1));
vel=read(fp2,-1,3);
file('close',fp2);
flag=vel(:,1);
v=vel(:,2);
plot2d(flag,v,style=2)
xtitle(['Alternative Overburden Velocities: H='+h1txt+'(m)'],"Station","Velocity (m/s)")
xgrid(3)
