//   |------------------------------------------------------------|
//   | $Id: delaytm.sci,v 1.6 2009-05-08 22:03:48 pm Exp $        |
//   | Basic Seismic Utilities (BSU)                              |
//   | LANGUAGE: Scilab                                           |
//   | AUTHOR: P. MICHAELS                                        |
//   | FUNCTION NAME: delaytm.sci                                 |
//   | DIRECTORY: BSUROOT/Scilab                                  |
//   |  revised for scilab 4.0                                    |
//   |------------------------------------------------------------|
//   | purpose: to compute delay time solution                    |
//   |   (normal refraction shooting, common shot gathers)        |
//   |------------------------------------------------------------|
//   | Scilab is an interpreter useful for scientifc calculations.|
//   | It is similar to MATLAB and may be obtained free from      |
//   | http://www-rocq.inria.fr/scilab                            |
//   | Scilab is available for many platforms, including LINUX    |
//   |------------------------------------------------------------|
//   |  NOTICE:                                                   |
//   |  Copyright (c) 2009 Paul Michaels                          |
//   |  <pm@cgiss.boisestate.edu>                                 |
//   |  This program is free software; you can                    |
//   |  redistribute it and/or modify it under the terms          |
//   |  of the GNU General Public License as published            |
//   |  by the Free Software Foundation; either version           |
//   |  3 of the License, or (at your option) any later           |
//   |  version.  This program is distributed in the              |
//   |  hope that it will be useful, but                          |
//   |  WITHOUT ANY WARRANTY; without even the implied            |
//   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
//   |  PARTICULAR PURPOSE.  See the GNU General Public           |
//   |  License for more details.                                 |
//   |  You should have received a copy of the GNU                |
//   |  General Public License along with this program;           |
//   |  if not, write to the Free Software Foundation,            |
//   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
//   |------------------------------------------------------------|
clear
ingfile=xgetfile('G*',title='Choose G=system matrix file ')
indfile=xgetfile('D*',title='Choose d=data vector file ')
inefile=xgetfile('E*',title='Choose e=elevations file ')

//read data
fp1=file('open',indfile,'old');
D=read(fp1,-1,2);
d=D(:,1);
flag=D(:,2);
file('close',fp1);

//read elevations
fp1=file('open',inefile,'old');
E=read(fp1,-1,3);
elv=E(:,3);
chan=E(:,1);
recv=E(:,2);
file('close',fp1);

nvarbl=evstr(x_dialog(['# Number of shots ? '],['1']));
nshot=nvarbl(1);

//assume 48 channels
//nchanl=48;
clear nvarlb;
nvarbl=evstr(x_dialog(['# Number of Channels? '],['48']));
nchanl=nvarbl(1);
ncols=nshot+nchanl+1;
//read system matrix 
fp1=file('open',ingfile,'old');
G=read(fp1,-1,ncols);
file('close',fp1);

[N,M]=size(G);
//nchanl=input('Enter number of channels ')
//nshot=input('Enter Number of Shots = ')
imax=nshot+nchanl+1;

//relative weighting matrix
pi2=3.1415926/2.;
// wsmooth=input(' Enter Smoothness Weight 0<W<1 ')
nvarbl=evstr(x_dialog(['Enter Smoothness Weight 0<W<1 '],['.10']));
wsmooth=nvarbl(1);
wsmtxt=sprintf("%f",wsmooth);
wfit=1-wsmooth;
for j=(nshot+1):(M-1)
for k=(nshot+1):M
W(j,k)=0;
end
W(j,j+1)=1;
W(j,j)=-1;
end
W(M,M)=1;
for j=1:(M-1)
W(M,j)=0;
end

// Compute Smooth Inverse
H=inv(wsmooth*W'*W+wfit*G'*G)*wfit*G';

m=H*d;

for i=1:nshot
dltsht(i)=m(i);
end
k=1;
for i=(nshot+1):(imax-1)
dltrcv(k)=m(i);
k=k+1;
end
v2=1/(m(imax));
v2txt=sprintf("%d",ceil(v2));
disp(' Refractor Velocity');
ceil(v2)

disp(' Shot Delay Times');
dltsht

//error analysis
cdd=eye(N,N);
sigmat=.001;  // pics +/- uncertainty, 1 std dev (68% conf)
for j=1:N
cdd(j,j)=cdd(j,j)*sigmat*sigmat;
end
cm=H*cdd*H';
sigmaslow=sqrt(cm(imax,imax));
sigmav2=(1/(m(imax)-sigmaslow)-1/(m(imax)+sigmaslow) )/2;
k=0;
for j=(nshot+1):(imax-1)
k=k+1;
errdt(k)=sqrt(cm(j,j));
end

//  plot delay times
jrecv(1:length(dltrcv))=recv(1:length(dltrcv));
plot(jrecv,dltrcv);
xmax=max(jrecv); xmin=min(jrecv);
ymax=max(dltrcv); ymin=min(dltrcv);

//label x-axis
labelx="Receiver Station Number";
//label y-axis
labely="Delay Time";
xtitle(['Receiver Delay Times (V2='+v2txt+')'],labelx,labely);

disp('writing delay times to disk .  .  .');
fp2=file('open',['dtimes.dat'],'unknown');
AA=[jrecv, dltrcv];
write(fp2,AA,'(2x,f10.2,2x,f10.4)');
file('close',fp2)

// v1=input(' Enter overburden velocity = ')
nvarbl=evstr(x_dialog(['Overburden Velocity '],['500']));
v1=nvarbl(1);
//plot structural interpretation (overburden velocity constant)
xbasc();
theta=asin(v1/v2);
cs=cos(theta);
h=(v1/cs)*dltrcv;
errh=(v1/cs)*errdt;  //error bars depth
NN=length(h);
for j=1:NN
e(j)=elv(j);
end
wx=e-h;

disp('writing errorbars to disk .  .  .');
fp2=file('open',['ebars.dat'],'unknown');
txt=sprintf(" Assumed V1=%d for Structural Interp. ",ceil(v1));
write(fp2,txt,'(a)');
txt=sprintf(" Pick Time Std_Dev=%f (68pct conf) ",sigmat);
write(fp2,txt,'(a)');
txt=sprintf(" V2=%d (+/-%d) 95 percent confidence",ceil(v2),ceil(1.96*sigmav2));
write(fp2,txt,'(a)');
txt=sprintf(" Error Bars 95pct conf ");
write(fp2,txt,'(a)');
txt=sprintf(" Receiver  Err_dt   Err_h) ");
write(fp2,txt,'(a)');
AA=[jrecv, errdt*1.96, errh ];
write(fp2,AA,'(1x,f5.0,2x,f10.4,2x,f10.2)');
file('close',fp2)

// plot structural solution
xmax=max(jrecv); xmin=min(jrecv);
ymax=max(e); ymin=min(wx);
frect=[xmin,ymin,xmax,ymax];

plot2d(jrecv,wx,style=1,rect=frect);
plot2d(jrecv,e,style=1,frameflag=0,axesflag=0);
v1txt=sprintf("%d",ceil(v1));

//label x-axis
labelx="Receiver Station Number";
//label y-axis
labely="Elevation";
xtitle(['Receiver Structure (V1='+v1txt+')'],labelx,labely);

disp('writing structure to disk .  .  .');
fp2=file('open',['struct.dat'],'unknown');
AA=[jrecv, wx, e];
write(fp2,AA,'(2x,f10.2,2x,f10.2,2x,f10.2)');
file('close',fp2)

//  h1=input(' Enter depth of refractor = ')
nvarbl=evstr(x_dialog(['Depth of refractor '],['10']));
h1=nvarbl(1);
xbasc();
//plot velocity interpretation (refractor depth=constant)
for jr=1:NN
vovb(jr)=v2/sqrt(1+ (dltrcv(jr)*(v2/h1)).^2);
end
// plot velocity solution
xmax=max(jrecv); xmin=min(jrecv);
ymax=max(vovb); ymin=min(vovb);
frect=[xmin,ymin,xmax,ymax];
//rect=[0,0,1.,1.];
//nax=[5,2,5,2];
//xsetech(rect,frect,"nn")
//plotframe(frect,nax);

plot2d(jrecv,vovb,style=1,rect=frect);
h1txt=sprintf("%d",ceil(h1));

//label x-axis
labelx="Receiver Station Number";
//label y-axis
labely="m/s";
xtitle(['Receiver V1 Velocity Solution (H1='+h1txt+')'],labelx,labely);

disp('writing velocity structure to disk   .  .');
fp2=file('open',['velocity.dat'],'unknown');
clear AA;
AA=[jrecv, vovb];
write(fp2,h1);
write(fp2,AA,'(2x,f10.2,2x,f10.2,2x,f10.2)');
file('close',fp2)

//calculated times
dcal=G*m;

//least square error
e=d-dcal;
emin=sqrt(e'*e/length(e));

//plot time fit
clear nvarbl;
nvarbl=evstr(x_dialog(['# Constraint Equations (if any) '],['1']));
nconst=nvarbl(1);
N2=(N-nconst);
// find break points between shots
jshot=1;
for j=1:N2
  if G(j,jshot)== 1 then
  kend(jshot)=j;
  else
  jshot=jshot+1;
  end
end

xdel();
[nr,nc]=size(d);
nr2=nr-nvarbl; // index of last time, before constraints
xmin=min(flag);
xmax=max(flag);
ymin=min(min(d(1:nr2)),min(dcal(1:nr2)));
ymax=max(max(d(1:nr2)),max(dcal(1:nr2)));
frect=[xmin,ymin,xmax,ymax];
kstart=1;
for k=1:nshot
if k==1 then
  plot2d(flag(kstart:kend(k)),d(kstart:kend(k)),style=-3,rect=frect);
else;
  plot2d(flag(kstart:kend(k)),d(kstart:kend(k)),style=-3,frameflag=0,axesflag=0);
end;  //endif
  plot2d(flag(kstart:kend(k)),dcal(kstart:kend(k)),style=2,frameflag=0,axesflag=0);
kstart=kend(k)+1;
end;  //next k
xtitle("0=Arrival Times and __=Solution","Station","Seconds");

disp('writing data times to disk   .  .');
fp2=file('open',['ttimes.dat'],'unknown');
write(fp2,[ceil(v1),ceil(v2)],'(2x,i5,2x,i5)');
write(fp2,nshot,'(2x,i5)');
for k=1:nshot
write(fp2,kend(k),'(1x,i5)');
end
clear AA;
kendd=length(flag)-nconst;
AA=[flag(1:kendd), d(1:kendd), dcal(1:kendd)];
write(fp2,AA,'(2x,f10.2,2x,f10.4,2x,f10.4)');
file('close',fp2)
