//   |------------------------------------------------------------|
//   | $Id: segpic.sci,v 1.3 2009-05-03 21:25:12 pm Exp $         |
//   | Basic Seismic Utilities (BSU)                              |
//   | LANGUAGE: Scilab                                           |
//   | AUTHOR: P. MICHAELS                                        |
//   | FUNCTION NAME: segpic.sci                                  |
//   | DIRECTORY: bsu-2.0.0/Scilab                                |
//   |  revised for scilab 4.0                                    |
//   |------------------------------------------------------------|
//   | purpose: read a BSEGY file pick first breaks with mouse.   |
//   |          write trace number and pic times to a file.       |
//   |------------------------------------------------------------|
//   | Scilab is an interpreter useful for scientifc calculations.|
//   | It is similar to MATLAB and may be obtained free from      |
//   | http://www-rocq.inria.fr/scilab                            |
//   | Scilab is available for many platforms, including LINUX    |
//   |------------------------------------------------------------|
//   |  NOTICE:                                                   |
//   |  Copyright (c) 2009 Paul Michaels                          |
//   |  <pm@cgiss.boisestate.edu>                                 |
//   |  This program is free software; you can                    |
//   |  redistribute it and/or modify it under the terms          |
//   |  of the GNU General Public License as published            |
//   |  by the Free Software Foundation; either version           |
//   |  2 of the License, or (at your option) any later           |
//   |  version.  This program is distributed in the              |
//   |  hope that it will be useful, but                          |
//   |  WITHOUT ANY WARRANTY; without even the implied            |
//   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
//   |  PARTICULAR PURPOSE.  See the GNU General Public           |
//   |  License for more details.                                 |
//   |  You should have received a copy of the GNU                |
//   |  General Public License along with this program;           |
//   |  if not, write to the Free Software Foundation,            |
//   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
//   |------------------------------------------------------------|

clear

//open data file with pointer fp1
filen=input('enter file name','s');

fp1=mopen(filen,'rb');

//get tmax
tmax=input('enter tmax');

//get clip factor
clipf=input('enter clip factor (1=no clip, 3 good choice)');

//move current position to byte 115
byte=mget(114,'uc',fp1);   //unsigned character uc

//read number of samples
npts=mget(1,'s',fp1);        //short integer
//read sample interval in microseconds
samin=mget(1,'s',fp1);       //short integer
fsamin=samin/1.E6;

//maximum sample number to plot
jmax=tmax/fsamin+1;
if jmax > npts then;
jmax=npts;
end;

//time vector
for j=1:npts;
tm(j)=(j-1)*fsamin;
end;

//close and reopen
mclose(fp1);
fp1=mopen(filen,'rb');


//--------------start loop
k=1;
while meof(fp1)==0;

//read header
byte=mget(240,'uc',fp1);   //unsigned character uc

if meof(fp1) ~=0 then; break; end;

//read a trace
data=mget(npts,'f',fp1);  //floating point
ymax=max(abs(data(1:jmax)))/clipf;
rect=[0,-ymax,tmax,ymax];

//plot the trace
plot2d(tm(1:jmax),data(1:jmax),2);
xtitle(string(k),"seconds","microvolts");
xgrid(3);
[b,xt,yt]=xclick();

xbasc();
pics(k)=xt;
k=k+1;
end;       //endwhile
xdel();  //close the graphic window
mclose(fp1);


//save pics
disp('writing pics to disk .  .  .')
fp2=file('open','pics.pic','unknown');
for k=1:length(pics);
fprintf(fp2,' %5.5d %8.4f \n',k,pics(k));
end;
file('close',fp2)



